/*-------------------------------------------------------------*/
/*  PROGTS2  written by Peter Edwards, September 1995          */
/*                                                             */
/*  MQSeries Test Program.                                     */
/*                                                             */
/*  Statement:  Licensed Materials - Property of IBM           */
/*                                                             */
/*              MA1C SupportPac                                */
/*              (c) Copyright IBM Corp. 1995, 2000.            */
/*                                                             */
/*              All rights reserved.                           */
/*                                                             */
/*              U.S. Government Users Restricted Rights - use, */
/*              duplication or disclosure restricted by GSA    */
/*              ADP Schedule Contract with IBM Corp.           */
/*-------------------------------------------------------------*/
/*  Program Description:                                       */
/*                                                             */
/*  This program takes no parameters, but reads an input file  */
/*  from the standard input.                                   */
/*  The input file contains keywords (with associated values)  */
/*  which are used to construct an MQSeries message with an    */
/*  MQIIH header followed by a single segment IMS format       */
/*  message (LLZZ<Data>), i.e. a message that can be processed */
/*  by the 'MQSeries for MVS' IMS Bridge.                      */
/*  This message is MQPUT to the specified target MQSeries     */
/*  Queue.                                                     */
/*                                                             */
/*                                                             */
/*  Input:                                                     */
/*                                                             */
/*  Program input takes the form of one or more groups of      */
/*  keywords each of which describes the properties and        */
/*  contents of an MQIIH message.                              */
/*  Each group of keywords is preceded by keyword:             */
/*    TESTSTART  <Optional Comment>                            */
/*  and ends with keyword:                                     */
/*    TESTEND    <Optional Comment>                            */
/*                                                             */
/*  Between TESTSTART and TESTEND, the following keywords are  */
/*  recognised:                                                */
/*    DEBUGLEVEL           0|1  0=minimum 1=full report detail */
/*    NOOFMESSAGES         n    Number of messages to MQPUT    */
/*    QMGRNAME             ccc  Target Queue Manager for MQPUT */
/*    QNAME                ccc  Target Queue for the MQPUT     */
/*    OPENOPTIONS          n    MQOPEN Options (MQOO_*) as a   */
/*                              decimal number. Multiple       */
/*                              OPENOPTIONS are added together */
/*    MQMDREPORT           n    The MQMD.Report                */
/*    MQMDMSGTYPE          n    The MQMD.MsgType               */
/*    MQMDEXPIRY           n    The MQMD.Expiry                */
/*    MQMDFEEDBACK         n    The MQMD.Feedback              */
/*    MQMDENCODING         n    The MQMD.Encoding              */
/*    MQMDCODEDCHARSETID   n    The MQMD.CodedCharSetId        */
/*    MQMDFORMAT           ccc  The MQMD.Format                */
/*    MQMDPRIORITY         n    The MQMD.Priority              */
/*    MQMDPERSISTENCE      n    The MQMD.Persistence           */
/*    MQMDMSGID            ccc  The MQMD.MsgId                 */
/*    MQMDCORRELID         ccc  The MQMD.CorrelId              */
/*    MQMDBACKOUTCOUNT     n    The MQMD.BackoutCount          */
/*    MQMDREPLYTOQUEUE     ccc  The MQMD.ReplyToQ              */
/*    MQMDREPLYTOQMGR      ccc  The MQMD.ReplyToQMgr           */
/*    MQMDUSERIDENTIFIER   ccc  The MQMD.UserIdentifier        */
/*    MQMDACCOUNTINGTOKEN  ccc  The MQMD.AccountingToken       */
/*    MQMDAPPLIDENTITYDATA ccc  The MQMD.ApplIdentityData      */
/*    MQMDPUTAPPLTYPE      n    The MQMD.PutApplType           */
/*    MQMDPUTAPPLNAME      ccc  The MQMD.PutApplName           */
/*    MQMDPUTDATE          ccc  The MQMD.PutDate               */
/*    MQMDPUTTIME          ccc  The MQMD.PutTime               */
/*    MQMDAPPLORIGINDATA   ccc  The MQMD.ApplOriginData        */
/*    MQPMOOPTIONS         n    Put msg options (MQPMO_*) as a */
/*                              decimal number. Multiple       */
/*                              MQPMOOPTIONS are added together*/
/*    MESSAGELENGTH        n    Total length of message        */
/*                              (including MQIIH length)       */
/*    MESSAGEDATA          ccc  Message data to appear after   */
/*                              the MQIIH and LLZZ             */
/*    IMSENCODING          n    The MQIIH.Encoding             */
/*    IMSCODEDCHARSETID    n    The MQIIH.CodedCharSetId       */
/*    IMSFORMAT            ccc  The MQIIH.Format               */
/*    IMSFLAGS             n    The MQIIH.Flags                */
/*    IMSLTERMOVERRIDE     ccc  The MQIIH.LTermOverride        */
/*    IMSMFSMAPNAME        ccc  The MQIIH.MFSMapName           */
/*    IMSREPLYTOFORMAT     ccc  The MQIIH.ReplyToFormat        */
/*    IMSAUTHENTICATOR     ccc  The MQIIH.Authenticator        */
/*    IMSTXNINSTANCEID     ccc  The MQIIH.TranInstanceId       */
/*    IMSTRANSTATE         c    The MQIIH.TranState            */
/*    IMSCOMMITMODE        c    The MQIIH.CommitMode           */
/*    IMSSECURITYSCOPE     c    The MQIIH.SecurityScope        */
/*    IMSRESERVED          c    The MQIIH.Reserved             */
/*    IMSLL                n    Decimal LL value               */
/*    IMSZZ                n    Decimal ZZ value               */
/*                                                             */
/*  Key to the above:                                          */
/*         n   = a decimal number                              */
/*         c   = a single character                            */
/*         ccc = a character string                            */
/*                                                             */
/*                                                             */
/*  Output:                                                    */
/*                                                             */
/*  Output messages are written to standard output (using      */
/*  printf).  The detail level of the output messages is       */
/*  determined by the setting of DEBUGLEVEL.                   */
/*                                                             */
/*                                                             */
/*  Usage Notes:                                               */
/*                                                             */
/*  1.Anything other than the above keywords is ignored, but   */
/*    beware of placing comments between keywords: a keyword   */
/*    is detected if it appears anywhere within an input line. */
/*                                                             */
/*  2.Except for MESSAGEDATA, anything following a keyword     */
/*    value is treated as a comment, provided at least one     */
/*    blank separates the comment from the value.              */
/*                                                             */
/*  3.If a keyword is not present, or a keyword has no value,  */
/*    where applicable the MQSeries defined default value is   */
/*    used. The following keywords have no defaults and must   */
/*    be specified:                                            */
/*       QMGRNAME, QNAME, MESSAGELENGTH, IMSLL, IMSZZ          */
/*                                                             */
/*  4.The normal IMS message format is LLZZ<Trancode><Data>,   */
/*    Data in the MESSAGEDATA field is placed immediately      */
/*    after LLZZ, so MESSAGEDATA will normally need to start   */
/*    with the required 8-byte Trancode, followed by any user  */
/*    data.                                                    */
/*    MESSAGEDATA will be padded with blanks up to 60 bytes    */
/*    and will then be followed by uninitialised data up to    */
/*    the required total message size (as defined by the       */
/*    MESSAGELENGTH field).                                    */
/*                                                             */
/*  5.MQSeries constants (as defined in cmqc.h) are NOT        */
/*    recognised as valid keyword values. Instead the decimal  */
/*    numeric or character value must be used, e.g. :-         */
/*    Valid                   Invalid                          */
/*    -----                   -------                          */
/*    MQMDFORMAT  MQIMS       MQMDFORMAT  MQFMT_IMS            */
/*    MQMDMSGTYPE 8           MQMDMSGTYPE MQMT_DATAGRAM        */
/*    OPENOPTIONS 2048        OPENOPTIONS MQOO_SET_ALL_CONTEXT */
/*                                                             */
/*  6.For the message to be accepted by the IMS Bridge,        */
/*    MESSAGELENGTH must equal LL + the size of the MQIIH      */
/*    header, i.e. LL + 84.                                    */
/*    The value of LL (field IMSLL) should be equal to the     */
/*    length of the data in the MESSAGEDATA field (which       */
/*    includes an 8-byte transaction code) plus 4-bytes for    */
/*    the length of LLZZ.                                      */
/*                                                             */
/*                                                             */
/*  Processing:                                                */
/*                                                             */
/*  This program should run on any platform where MQSeries     */
/*  is supported, provided it is linked with the correct       */
/*  MQSeries library.                                          */
/*-------------------------------------------------------------*/
/*  Constants.                                                 */
/*-------------------------------------------------------------*/

#define CURRPROG "PROGTS2  "

/*-------------------------------------------------------------*/
/*  C library header files.                                    */
/*-------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <cmqc.h>

/*-------------------------------------------------------------*/
/*  Input file structure                                       */
/*-------------------------------------------------------------*/

typedef struct tag_infile {
        char             fieldname??(20??);
        char             fieldvalue??(60??);
       } INFILE;

/*-------------------------------------------------------------*/
/*  MQIIH definitions (if not already defined in cmqc.h)       */
/*-------------------------------------------------------------*/
#ifndef MQIIH_DEFAULT
typedef struct tagMQIIH {
  MQCHAR4   StrucId;         /* Structure identifier */
  MQLONG    Version;         /* Structure version number */
  MQLONG    StrucLength;     /* Length of MQIIH structure */
  MQLONG    Encoding;        /* Reserved */
  MQLONG    CodedCharSetId;  /* Reserved */
  MQCHAR8   Format;          /* Format name */
  MQLONG    Flags;           /* Reserved */
  MQCHAR8   LTermOverride;   /* Logical terminal override */
  MQCHAR8   MFSMapName;      /* Message format services map name */
  MQCHAR8   ReplyToFormat;   /* Format name of reply message */
  MQCHAR8   Authenticator;   /* RACF password or pass ticket */
  MQBYTE16  TranInstanceId;  /* Transaction instance id */
  MQCHAR    TranState;       /* Transaction state */
  MQCHAR    CommitMode;      /* Commit mode */
  MQCHAR    SecurityScope;   /* Security scope */
  MQCHAR    Reserved;        /* Reserved */
 } MQIIH;

#define MQIIH_VERSION_1 1L
#define MQIIH_LENGTH_1  84L
#define MQIIH_NONE      0L
#define MQFMT_IMS       "MQIMS   "
#define MQITS_NOT_IN_CONVERSATION ' '
#define MQISS_CHECK     'C'
#define MQICM_COMMIT_THEN_SEND '0'
#endif
/*-------------------------------------------------------------*/
/*  Function prototypes.                                       */
/*-------------------------------------------------------------*/

int processtest (char *);
int putmessage(void);
void initstor(void);

     int        debuglvl;
     INFILE     Inputrecord;
     MQIIH      IMSheader;
     MQMD       mqmd;
     MQOD       mqod;
     MQPMO      mqpmo;
     MQHCONN    Connhandle;
     MQHOBJ     Objhandle;
     MQLONG     CompCode, Reason, Openoptions, CloseOptions;
     MQLONG     Thisopenoption, Thisreportoption, Thisputoption;
     MQLONG     Thisputappltype;
     MQLONG     MessageLength;
     char       Inbuff??(81??), buffnum??(60??), buffchar??(60??);
     char       MessageData??(60??);
     MQCHAR48   Qmgrname, Qname;
     char       *ptr, *endptr, *malloc_addr, *IMSLL_addr, *IMSZZ_addr;
     char       *MessageData_addr;
     long       Noofmessages, IMSLLlong, IMSZZlong;
     short      IMSLL, IMSZZ;
     char       QueueOpened??(49??);
     long       Duplicate;

int main(void)
{

     int        i, j;
     initstor ();
/*-------------------------------------------------------------*/
/*  Get records from the input file.                           */
/*-------------------------------------------------------------*/

     printf ("Program started \n");
     while (gets(Inbuff) != NULL)
     {
      printf ("Record read: %s\n",Inbuff);
      /*-------------------------------------------------------*/
      /* Find the first/next TESTSTART                         */
      /*-------------------------------------------------------*/
      ptr = strstr(Inbuff,"TESTSTART");
      if (ptr != NULL)
         {
         ptr = strstr(Inbuff,"DUPLICATE=");
         if (ptr != NULL)
         {
           ptr += 10;
           Duplicate = strtol(ptr,&endptr,10);
           if (Duplicate < 1)
             Duplicate = 1;
           printf("Test will be duplicated %d times\n",Duplicate);
         }
         else Duplicate = 1;

         while (gets(Inbuff) != NULL)
            {
            if (debuglvl != 0)
               printf ("Record read: %s\n",Inbuff);
            /*-------------------------------------------------*/
            /* Process records until TESTEND                   */
            /*-------------------------------------------------*/
            ptr = strstr(Inbuff,"TESTEND");
            if (ptr != NULL)
               break;
            i = processtest((char *)Inbuff);
            if (i == 1)
               break;
            }
         /*----------------------------------------------------*/
         /* MQPUT the msg(s), re-initialise the variables then */
         /* backround to look for another TESTSTART.           */
         /*----------------------------------------------------*/
         for (j=0; j<Duplicate; j++)
         {
           if ((debuglvl != 0) && (Duplicate != 1))
              printf ("*** Duplicate iteration # %d ***\n",j+1);
           i = putmessage();
           if (i == 1)
             break;
         }
         initstor();
         }
     }
     return(0);
}

/*-------------------------------------------------------------*/
/*  Initialise storage, using MQ defaults where appropriate.   */
/*-------------------------------------------------------------*/
void initstor(void)
{
     debuglvl = 0;
     Noofmessages = 1;
     memset (Qmgrname,'\0',48);
     memset (Qname,'\0',48);
     Openoptions = 0;
     CloseOptions = MQCO_NONE;
     memcpy (mqmd.StrucId, MQMD_STRUC_ID, 4);
     mqmd.Version = MQMD_VERSION_1;
     mqmd.Report = MQRO_NONE;
     mqmd.MsgType = MQMT_REQUEST;
     mqmd.Expiry = MQEI_UNLIMITED;
     mqmd.Feedback = MQFB_NONE;
     mqmd.Encoding = MQENC_NATIVE;
     mqmd.CodedCharSetId = MQCCSI_Q_MGR;
     memcpy (mqmd.Format, MQFMT_NONE, 8);
     mqmd.Priority = MQPRI_PRIORITY_AS_Q_DEF;
     mqmd.Persistence = MQPER_PERSISTENCE_AS_Q_DEF;
     memcpy (mqmd.MsgId, MQMI_NONE, 24);
     memcpy (mqmd.CorrelId, MQCI_NONE, 24);
     memset (mqmd.ReplyToQMgr,' ',48);
     memset (mqmd.ReplyToQ,' ',48);
     memset (mqmd.UserIdentifier,' ',12);
     memset (mqmd.AccountingToken,'\0',32);
     memset (mqmd.ApplIdentityData,' ',32);
     mqmd.PutApplType = MQAT_DEFAULT;
     memset (mqmd.PutApplName,' ',28);
     memset (mqmd.PutDate,' ',8);
     memset (mqmd.PutTime,' ',8);
     memset (mqmd.ApplOriginData,' ',4);
     memset (Inbuff, '\0', 81);
     memcpy (mqod.StrucId, MQOD_STRUC_ID, 4);
     mqod.Version = MQOD_VERSION_1;
     mqod.ObjectType = MQOT_Q;
     strcpy(mqod.DynamicQName,"VIC114.PROGTS2.*");
     memcpy (mqpmo.StrucId, MQPMO_STRUC_ID, 4);
     mqpmo.Version = MQPMO_VERSION_1;
     mqpmo.Options = MQPMO_NONE;
     memcpy (IMSheader.StrucId,"IIH ", 4);
     IMSheader.Version = MQIIH_VERSION_1;
     IMSheader.StrucLength = MQIIH_LENGTH_1;
     IMSheader.Encoding = MQENC_NATIVE;
     IMSheader.CodedCharSetId = MQCCSI_Q_MGR;
     memcpy (IMSheader.Format,MQFMT_NONE,8);
     IMSheader.Flags = MQIIH_NONE;
     memset (IMSheader.LTermOverride,' ',8);
     memset (IMSheader.MFSMapName,' ',8);
     memset (IMSheader.ReplyToFormat,' ',8);
     memset (IMSheader.Authenticator,' ',8);
     memset (IMSheader.TranInstanceId,'\0',16);
     IMSheader.TranState = MQITS_NOT_IN_CONVERSATION;
     IMSheader.CommitMode = MQICM_COMMIT_THEN_SEND;
     IMSheader.SecurityScope = MQISS_CHECK;
     IMSheader.Reserved = MQIIH_NONE;
}

/*-------------------------------------------------------------*/
/*  Connect to the Qmgr, open the queue and put the messages   */
/*  using the definitions obtained from the input file by      */
/*  processtest().                                             */
/*-------------------------------------------------------------*/
int putmessage(void)
{
     int  i;

     /*--------------------------------------------------------*/
     /* Connect to the target Queue Manager.                   */
     /*--------------------------------------------------------*/
     MQCONN (Qmgrname, &Connhandle, &CompCode, &Reason);
     if ((CompCode != MQCC_OK) && (Reason != MQRC_ALREADY_CONNECTED))
        {
        printf ("MQCONN failed, reason: %d\n",Reason);
        printf ("Qmgrname: %s\n",Qmgrname);
        return(1);
        }
     printf ("MQCONN completion code: %d\n",CompCode);
     printf ("MQCONN reason code: %d\n",Reason);

     memcpy (mqod.ObjectName, Qname, 48);
     memcpy (mqod.ObjectQMgrName, Qmgrname, 48);

     /*--------------------------------------------------------*/
     /* Open the target Queue.                                 */
     /*--------------------------------------------------------*/
     MQOPEN (Connhandle, &mqod, Openoptions, &Objhandle,
             &CompCode, &Reason);
     if (CompCode != MQCC_OK)
        {
        printf ("MQOPEN failed, reason: %d\n",Reason);
        return(1);
        }
     memcpy (QueueOpened, mqod.ObjectName, 48); /* Print the actual */
     QueueOpened??(48??) = '\0';                /* queue name opened*/
     printf ("MQOPEN: %s\n",QueueOpened);       /* ...              */
     printf ("MQOPEN completion code: %d\n",CompCode);
     printf ("MQOPEN reason code: %d\n",Reason);

     /*--------------------------------------------------------*/
     /* Get storage for the MQPUT message buffer, making sure  */
     /* there's enough space for at least the MQIIH header +   */
     /* LLZZ and the MessageData.                              */
     /*--------------------------------------------------------*/
     if (MessageLength < (sizeof(MessageData) + sizeof(IMSheader)
          + sizeof(IMSLL) + sizeof(IMSZZ)))
        malloc_addr = ((char *) malloc(
                         (sizeof(MessageData) + sizeof(IMSheader)
          + sizeof(IMSLL) + sizeof(IMSZZ))));
     else
        malloc_addr = ((char *) malloc(MessageLength));
     /*--------------------------------------------------------*/
     /* Only proceed if malloc() was successful.               */
     /* Copy the MQIIH (IMSheader), LL, ZZ and MessageData into*/
     /* the malloc'ed storage.                                 */
     /*--------------------------------------------------------*/
     if (malloc_addr != NULL)
        {
        IMSLL_addr = malloc_addr + sizeof(IMSheader);
        IMSZZ_addr = IMSLL_addr + sizeof(IMSLL);
        MessageData_addr = IMSZZ_addr + sizeof(IMSZZ);
        memcpy (malloc_addr, &IMSheader, sizeof(IMSheader));
        memcpy (IMSLL_addr, &IMSLL, sizeof(IMSLL));
        memcpy (IMSZZ_addr, &IMSZZ, sizeof(IMSZZ));
        strcpy (MessageData_addr, MessageData);
        printf ("Message data: <%.60s>\n", MessageData);
        /*-----------------------------------------------------*/
        /* MQPUT the required number of messages.              */
        /*-----------------------------------------------------*/
        for (i=0; i < Noofmessages; i++)
        {
        MQPUT  (Connhandle, Objhandle, &mqmd, &mqpmo, MessageLength,
                malloc_addr, &CompCode, &Reason);
        if (CompCode != MQCC_OK)
           {
           printf ("MQPUT failed, reason: %d\n",Reason);
           free (malloc_addr);
           return(1);
           }
        printf ("MQPUT  completion code: %d\n",CompCode);
        printf ("MQPUT  reason code: %d\n",Reason);
        }
     }

     /*--------------------------------------------------------*/
     /* Close the Queue.                                       */
     /*--------------------------------------------------------*/
     MQCLOSE (Connhandle, &Objhandle, CloseOptions,
             &CompCode, &Reason);
     if (CompCode != MQCC_OK)
        {
        printf ("MQCLOSE failed, reason: %d\n",Reason);
        free (malloc_addr);
        return(1);
        }
     printf ("MQCLOSE completion code: %d\n",CompCode);
     printf ("MQCLOSE reason code: %d\n",Reason);

     /*--------------------------------------------------------*/
     /* Commit the messages.                                   */
     /*--------------------------------------------------------*/
     MQCMIT (Connhandle,
             &CompCode, &Reason);
     if (CompCode != MQCC_OK)
        {
        printf ("MQCMIT  failed, reason: %d\n",Reason);
        free (malloc_addr);
        return(1);
        }
     printf ("MQCMIT completion code: %d\n",CompCode);
     printf ("MQCMIT reason code: %d\n",Reason);

     free (malloc_addr);
     return(0);
}

/*-------------------------------------------------------------*/
/*  Process an input record, looking for one of the recognised */
/*  keywords.                                                  */
/*-------------------------------------------------------------*/
int processtest(char *Inbuff)
{

     int   i=0, j, k=0;
     char  Inputfieldname??(81??);

     memset (Inputfieldname, '\0', 81);
     /*--------------------------------------------------------*/
     /* Take a copy of the input record.                       */
     /*--------------------------------------------------------*/
     memcpy ((char *)Inputfieldname, Inbuff, 80);
     ptr = Inputfieldname;
     /*--------------------------------------------------------*/
     /* Skip over any leading blanks...                        */
     /*--------------------------------------------------------*/
     while ((*ptr == ' ') &&
                  (ptr <= Inputfieldname+80))
             {
                  ptr++;
                  i++;
                }
     /*--------------------------------------------------------*/
     /* ...then find the end of the keyword                    */
     /*--------------------------------------------------------*/
     while ((*ptr != ' ') &&
                  (ptr <= Inputfieldname+80))
             {
                  ptr++;
                   k++;
                }
     /*--------------------------------------------------------*/
     /* Translate the keyword to upper case.                   */
     /*--------------------------------------------------------*/
     for (j=0; j<k; j++)
        {
        Inputfieldname??(j+i??) = toupper(Inputfieldname??(j+i??));
        }
     if (debuglvl != 0)
        printf ("Input record: %s\n", Inputfieldname);
     /*--------------------------------------------------------*/
     /* Make sure we get a TESTEND before another TESTSTART.   */
     /*--------------------------------------------------------*/
     ptr = strstr(Inputfieldname,"TESTSTART");
     if (ptr != NULL)
       /* input file error - we need a TESTEND record before another
          TESTSTART */
         return(1);

     /*--------------------------------------------------------*/
     /* Search the input record for each of the valid keywords.*/
     /*                                                        */
     /* When found, the next non-blank character is the start  */
     /* of the keyword value. The first null or blank is the   */
     /* end of the value.                                      */
     /* Numeric values are loaded, one char at a time, into    */
     /* array buffnum, then converted to a long by strtol()    */
     /* and placed in the appropriate variable.                */
     /* Character values are loaded, one char at a time, into  */
     /* buffchar, then memcpy()'ed to the appropriate character*/
     /* field.                                                 */
     /*--------------------------------------------------------*/
     ptr = strstr(Inputfieldname,"DEBUGLEVEL");
     if (ptr != NULL)
       /* need to check value on record - values 0 (no) or 1 (detail) */
         {
         memset (buffnum, '\0', sizeof(buffnum));
         ptr = ptr + 11;
         if (ptr <= Inputfieldname+80)
            {
            /*  jump over any spaces   */
            while ((*ptr == ' ') &&
                  (ptr <= Inputfieldname+80))
                   ptr++;
            for (i=0;
                (ptr <= Inputfieldname+80) &&
                (*ptr != '\0') &&
                (*ptr != ' '); i++, ptr++)
                 buffnum??(i??) = *ptr;
            if (buffnum??(0??) != '\0')
                   debuglvl = strtol(buffnum,&endptr,10);
            }
         else
            debuglvl = 0;
         if (debuglvl != 0)
            printf ("Debuglvl: %i\n",debuglvl);
         return(0);
         }

     ptr = strstr(Inputfieldname,"NOOFMESSAGES");
     if (ptr != NULL)
       /* we have a field containing number of messages */
        {
         ptr = ptr + 12;
         memset (buffnum, '\0', sizeof(buffnum));
         if (ptr <= Inputfieldname+80)
            {
            while ((*ptr == ' ') &&
                  (ptr <= Inputfieldname+80))
                   ptr++;
            for (i=0;
                (ptr <= Inputfieldname+80) &&
                (*ptr != '\0') &&
                (*ptr != ' '); i++, ptr++)
                 buffnum??(i??) = *ptr;
            if (buffnum??(0??) != '\0')
               Noofmessages = strtol(buffnum,&endptr,10);
            }
         if (debuglvl != 0)
            printf ("No of messages: %i\n",Noofmessages);
         return(0);
        }

     ptr = strstr(Inputfieldname,"QMGRNAME");
     if (ptr != NULL)
       /* we have a field containing queue manager name */
        {
         ptr = ptr + 8;
         memset (buffchar, '\0', 60);
         if (ptr <= Inputfieldname+80)
            {
            while ((*ptr == ' ') &&
                  (ptr <= Inputfieldname+80))
                   ptr++;
            for (i=0;
                (ptr <= Inputfieldname+80) &&
                (*ptr != '\0') &&
                (*ptr != ' '); i++, ptr++)
                 buffchar??(i??) = *ptr;
            if (buffchar??(0??) != '\0')
               {
               memcpy (Qmgrname, buffchar, 48);
               }
            }
         if (debuglvl != 0)
            printf ("Qmgrname: %s\n",Qmgrname);
         return(0);
         }

     ptr = strstr(Inputfieldname,"QNAME");
     if (ptr != NULL)
       /* we have a field containing queue manager name */
        {
         ptr = ptr + 5;
         memset (buffchar, '\0', 60);
         if (ptr <= Inputfieldname+80)
            {
            while ((*ptr == ' ') &&
                  (ptr <= Inputfieldname+80))
                   ptr++;
            for (i=0;
                (ptr <= Inputfieldname+80) &&
                (*ptr != '\0') &&
                (*ptr != ' '); i++, ptr++)
                 buffchar??(i??) = *ptr;
            if (buffchar??(0??) != '\0')
               memcpy (Qname, buffchar, 48);
            }
         if (debuglvl != 0)
            printf ("Qname: %s\n",Qname);
         return(0);
         }

     ptr = strstr(Inputfieldname,"OPENOPTIONS");
     if (ptr != NULL)
       /* we have a field containing open options */
       /* nb there may be more than one - add options together */
        {
         ptr = ptr + 11;
         memset (buffnum, '\0', sizeof(buffnum));
         if (ptr <= Inputfieldname+80)
            {
            while ((*ptr == ' ') &&
                  (ptr <= Inputfieldname+80))
                   ptr++;
            for (i=0;
                (ptr <= Inputfieldname+80) &&
                (*ptr != '\0') &&
                (*ptr != ' '); i++, ptr++)
                 buffnum??(i??) = *ptr;
            if (buffnum??(0??) != '\0')
              {
               Thisopenoption = strtol(buffnum,&endptr,10);
               Openoptions = Openoptions + Thisopenoption;
              }
            }
         if (debuglvl != 0)
            printf ("Openoptions: %i\n",Openoptions);
          return(0);
         }

     ptr = strstr(Inputfieldname,"MQMDREPORT");
     if (ptr != NULL)
       /* we have a field containing report options */
       /* nb there may be more than one - add options together */
        {
         ptr = ptr + 10;
         memset (buffnum, '\0', sizeof(buffnum));
         if (ptr <= Inputfieldname+80)
            {
            while ((*ptr == ' ') &&
                  (ptr <= Inputfieldname+80))
                   ptr++;
            for (i=0;
                (ptr <= Inputfieldname+80) &&
                (*ptr != '\0') &&
                (*ptr != ' '); i++, ptr++)
                 buffnum??(i??) = *ptr;
            if (buffnum??(0??) != '\0')
              {
               Thisreportoption = strtol(buffnum,&endptr,10);
               mqmd.Report = mqmd.Report + Thisreportoption;
              }
            }
         if (debuglvl != 0)
            printf ("Report options: %i\n",mqmd.Report);
          return(0);
         }

     ptr = strstr(Inputfieldname,"MQMDMSGTYPE");
     if (ptr != NULL)
       /* we have a field containing msgtype options */
        {
         ptr = ptr + 11;
         memset (buffnum, '\0', sizeof(buffnum));
         if (ptr <= Inputfieldname+80)
            {
            while ((*ptr == ' ') &&
                  (ptr <= Inputfieldname+80))
                   ptr++;
            for (i=0;
                (ptr <= Inputfieldname+80) &&
                (*ptr != '\0') &&
                (*ptr != ' '); i++, ptr++)
                 buffnum??(i??) = *ptr;
            if (buffnum??(0??) != '\0')
               mqmd.MsgType = strtol(buffnum,&endptr,10);
            }
         if (debuglvl != 0)
            printf ("MQMD message type: %i\n",mqmd.MsgType);
          return(0);
         }

     ptr = strstr(Inputfieldname,"MQMDEXPIRY");
     if (ptr != NULL)
       /* we have a field containing expiry */
        {
         ptr = ptr + 10;
         memset (buffnum, '\0', sizeof(buffnum));
         if (ptr <= Inputfieldname+80)
            {
            while ((*ptr == ' ') &&
                  (ptr <= Inputfieldname+80))
                   ptr++;
            for (i=0;
                (ptr <= Inputfieldname+80) &&
                (*ptr != '\0') &&
                (*ptr != ' '); i++, ptr++)
                 buffnum??(i??) = *ptr;
            if (buffnum??(0??) != '\0')
               mqmd.Expiry = strtol(buffnum,&endptr,10);
            }
         if (debuglvl != 0)
            printf ("MQMD expiry: %i\n",mqmd.Expiry);
          return(0);
         }

     ptr = strstr(Inputfieldname,"MQMDFEEDBACK");
     if (ptr != NULL)
       /* we have a field containing feedback */
        {
         ptr = ptr + 12;
         memset (buffnum, '\0', sizeof(buffnum));
         if (ptr <= Inputfieldname+80)
            {
            while ((*ptr == ' ') &&
                  (ptr <= Inputfieldname+80))
                   ptr++;
            for (i=0;
                (ptr <= Inputfieldname+80) &&
                (*ptr != '\0') &&
                (*ptr != ' '); i++, ptr++)
                 buffnum??(i??) = *ptr;
            if (buffnum??(0??) != '\0')
               mqmd.Feedback = strtol(buffnum,&endptr,10);
            }
         if (debuglvl != 0)
            printf ("MQMD feedback: %i\n",mqmd.Feedback);
          return(0);
         }

     ptr = strstr(Inputfieldname,"MQMDENCODING");
     if (ptr != NULL)
       /* we have a field containing encoding */
        {
         ptr = ptr + 12;
         memset (buffnum, '\0', sizeof(buffnum));
         if (ptr <= Inputfieldname+80)
            {
            while ((*ptr == ' ') &&
                  (ptr <= Inputfieldname+80))
                   ptr++;
            for (i=0;
                (ptr <= Inputfieldname+80) &&
                (*ptr != '\0') &&
                (*ptr != ' '); i++, ptr++)
                 buffnum??(i??) = *ptr;
            if (buffnum??(0??) != '\0')
               mqmd.Encoding = strtol(buffnum,&endptr,10);
            }
         if (debuglvl != 0)
            printf ("MQMD encoding: %i\n",mqmd.Encoding);
          return(0);
         }

     ptr = strstr(Inputfieldname,"MQMDCODEDCHARSETID");
     if (ptr != NULL)
       /* we have a field containing ccsid */
        {
         ptr = ptr + 18;
         memset (buffnum, '\0', sizeof(buffnum));
         if (ptr <= Inputfieldname+80)
            {
            while ((*ptr == ' ') &&
                  (ptr <= Inputfieldname+80))
                   ptr++;
            for (i=0;
                (ptr <= Inputfieldname+80) &&
                (*ptr != '\0') &&
                (*ptr != ' '); i++, ptr++)
                 buffnum??(i??) = *ptr;
            if (buffnum??(0??) != '\0')
               mqmd.CodedCharSetId = strtol(buffnum,&endptr,10);
            }
         if (debuglvl != 0)
            printf ("MQMD CCSid: %i\n",mqmd.CodedCharSetId);
          return(0);
         }

     ptr = strstr(Inputfieldname,"MQMDFORMAT");
     if (ptr != NULL)
       /* we have a field containing format */
        {
         ptr = ptr + 10;
         memset (buffchar, '\0', 60);
         if (ptr <= Inputfieldname+80)
            {
            while ((*ptr == ' ') &&
                  (ptr <= Inputfieldname+80))
                   ptr++;
            for (i=0;
                (ptr <= Inputfieldname+80) &&
                (*ptr != '\0') &&
                (*ptr != ' '); i++, ptr++)
                 buffchar??(i??) = *ptr;
            if (buffchar??(0??) != '\0')
               memcpy (mqmd.Format, buffchar, 8);
            }
         if (debuglvl != 0)
            printf ("MQMD Format: %s\n",mqmd.Format);
         return(0);
         }

     ptr = strstr(Inputfieldname,"MQMDPRIORITY");
     if (ptr != NULL)
       /* we have a field containing priority */
        {
         ptr = ptr + 12;
         memset (buffnum, '\0', sizeof(buffnum));
         if (ptr <= Inputfieldname+80)
            {
            while ((*ptr == ' ') &&
                  (ptr <= Inputfieldname+80))
                   ptr++;
            for (i=0;
                (ptr <= Inputfieldname+80) &&
                (*ptr != '\0') &&
                (*ptr != ' '); i++, ptr++)
                 buffnum??(i??) = *ptr;
            if (buffnum??(0??) != '\0')
               mqmd.Priority = strtol(buffnum,&endptr,10);
            }
         if (debuglvl != 0)
            printf ("MQMD Priority: %i\n",mqmd.Priority);
          return(0);
         }

     ptr = strstr(Inputfieldname,"MQMDPERSISTENCE");
     if (ptr != NULL)
       /* we have a field containing persistence */
        {
         ptr = ptr + 15;
         memset (buffnum, '\0', sizeof(buffnum));
         if (ptr <= Inputfieldname+80)
            {
            while ((*ptr == ' ') &&
                  (ptr <= Inputfieldname+80))
                   ptr++;
            for (i=0;
                (ptr <= Inputfieldname+80) &&
                (*ptr != '\0') &&
                (*ptr != ' '); i++, ptr++)
                 buffnum??(i??) = *ptr;
            if (buffnum??(0??) != '\0')
               mqmd.Persistence = strtol(buffnum,&endptr,10);
            }
         if (debuglvl != 0)
            printf ("MQMD Persistence: %i\n",mqmd.Persistence);
          return(0);
         }

     ptr = strstr(Inputfieldname,"MQMDMSGID");
     if (ptr != NULL)
       /* we have a field containing msgid */
        {
         ptr = ptr + 9;
         memset (buffchar, '\0', 60);
         if (ptr <= Inputfieldname+80)
            {
            while ((*ptr == ' ') &&
                  (ptr <= Inputfieldname+80))
                   ptr++;
            for (i=0;
                (ptr <= Inputfieldname+80) &&
                (*ptr != '\0') &&
                (*ptr != ' '); i++, ptr++)
                 buffchar??(i??) = *ptr;
            if (buffchar??(0??) != '\0')
               memcpy (mqmd.MsgId, buffchar, 24);
            }
         if (debuglvl != 0)
            printf ("MQMD Msgid: %s\n",mqmd.MsgId);
         return(0);
         }

     ptr = strstr(Inputfieldname,"MQMDCORRELID");
     if (ptr != NULL)
       /* we have a field containing correlid */
        {
         ptr = ptr + 12;
         memset (buffchar, '\0', 60);
         if (ptr <= Inputfieldname+80)
            {
            while ((*ptr == ' ') &&
                  (ptr <= Inputfieldname+80))
                   ptr++;
            for (i=0;
                (ptr <= Inputfieldname+80) &&
                (*ptr != '\0') &&
                (*ptr != ' '); i++, ptr++)
                 buffchar??(i??) = *ptr;
            if (buffchar??(0??) != '\0')
               memcpy (mqmd.CorrelId, buffchar, 24);
            }
         if (debuglvl != 0)
            printf ("MQMD Correlid: %s\n",mqmd.CorrelId);
         return(0);
         }

     ptr = strstr(Inputfieldname,"MQMDBACKOUTCOUNT");
     if (ptr != NULL)
       /* we have a field containing backout count */
        {
         ptr = ptr + 16;
         memset (buffnum, '\0', sizeof(buffnum));
         if (ptr <= Inputfieldname+80)
            {
            while ((*ptr == ' ') &&
                  (ptr <= Inputfieldname+80))
                   ptr++;
            for (i=0;
                (ptr <= Inputfieldname+80) &&
                (*ptr != '\0') &&
                (*ptr != ' '); i++, ptr++)
                 buffnum??(i??) = *ptr;
            if (buffnum??(0??) != '\0')
               mqmd.BackoutCount = strtol(buffnum,&endptr,10);
            }
         if (debuglvl != 0)
            printf ("MQMD BackoutCount: %i\n",mqmd.BackoutCount);
          return(0);
         }

     ptr = strstr(Inputfieldname,"MQMDREPLYTOQUEUE");
     if (ptr != NULL)
       /* we have a field containing queue manager name */
        {
         ptr = ptr + 16;
         memset (buffchar, '\0', 60);
         if (ptr <= Inputfieldname+80)
            {
            while ((*ptr == ' ') &&
                  (ptr <= Inputfieldname+80))
                   ptr++;
            for (i=0;
                (ptr <= Inputfieldname+80) &&
                (*ptr != '\0') &&
                (*ptr != ' '); i++, ptr++)
                 buffchar??(i??) = *ptr;
            if (buffchar??(0??) != '\0')
               memcpy (mqmd.ReplyToQ, buffchar, 48);
            }
         if (debuglvl != 0)
            printf ("MQMD Replytoq: %s\n",mqmd.ReplyToQ);
         return(0);
         }

     ptr = strstr(Inputfieldname,"MQMDREPLYTOQMGR");
     if (ptr != NULL)
       /* we have a field containing replyto queue manager name */
        {
         ptr = ptr + 15;
         memset (buffchar, '\0', 60);
         if (ptr <= Inputfieldname+80)
            {
            while ((*ptr == ' ') &&
                  (ptr <= Inputfieldname+80))
                   ptr++;
            for (i=0;
                (ptr <= Inputfieldname+80) &&
                (*ptr != '\0') &&
                (*ptr != ' '); i++, ptr++)
                 buffchar??(i??) = *ptr;
            if (buffchar??(0??) != '\0')
               memcpy (mqmd.ReplyToQMgr, buffchar, 48);
            }
         if (debuglvl != 0)
            printf ("MQMD Replytoqmgr: %s\n",mqmd.ReplyToQMgr);
         return(0);
         }


     ptr = strstr(Inputfieldname,"MQMDUSERIDENTIFIER");
     if (ptr != NULL)
       /* we have a field containing useridentifier */
        {
         ptr = ptr + 18;
         memset (buffchar, ' ', 60);
         if (ptr <= Inputfieldname+80)
            {
            while ((*ptr == ' ') &&
                  (ptr <= Inputfieldname+80))
                   ptr++;
            for (i=0;
                (ptr <= Inputfieldname+80) &&
                (*ptr != '\0') &&
                (*ptr != ' '); i++, ptr++)
                 buffchar??(i??) = *ptr;
            if (buffchar??(0??) != '\0')
               memcpy (mqmd.UserIdentifier, buffchar, 12);
            }
         if (debuglvl != 0)
            printf ("MQMD Userid: %s\n",mqmd.UserIdentifier);
         return(0);
         }

     ptr = strstr(Inputfieldname,"MQMDACCOUNTINGTOKEN");
     if (ptr != NULL)
       /* we have a field containing account token  */
        {
         ptr = ptr + 19;
         memset (buffchar, '\0', 60);
         if (ptr <= Inputfieldname+80)
            {
            while ((*ptr == ' ') &&
                  (ptr <= Inputfieldname+80))
                   ptr++;
            for (i=0;
                (ptr <= Inputfieldname+80) &&
                (*ptr != '\0') &&
                (*ptr != ' '); i++, ptr++)
                 buffchar??(i??) = *ptr;
            if (buffchar??(0??) != '\0')
               memcpy (mqmd.AccountingToken, buffchar, 32);
            }
         if (debuglvl != 0)
            printf ("MQMD Account token: %s\n",mqmd.AccountingToken);
         return(0);
         }

     ptr = strstr(Inputfieldname,"MQMDAPPLIDENTITYDATA");
     if (ptr != NULL)
       /* we have a field containing appl identity  */
        {
         ptr = ptr + 20;
         memset (buffchar, '\0', 60);
         if (ptr <= Inputfieldname+80)
            {
            while ((*ptr == ' ') &&
                  (ptr <= Inputfieldname+80))
                   ptr++;
            for (i=0;
                (ptr <= Inputfieldname+80) &&
                (*ptr != '\0') &&
                (*ptr != ' '); i++, ptr++)
                 buffchar??(i??) = *ptr;
            if (buffchar??(0??) != '\0')
               memcpy (mqmd.ApplIdentityData, buffchar, 32);
            }
         if (debuglvl != 0)
            printf ("MQMD Appl Ident Data: %s\n",mqmd.ApplIdentityData);
         return(0);
         }

     ptr = strstr(Inputfieldname,"MQMDPUTAPPLNAME");
     if (ptr != NULL)
       /* we have a field containing putapplname  */
        {
         ptr = ptr + 15;
         memset (buffchar, '\0', 60);
         if (ptr <= Inputfieldname+80)
            {
            while ((*ptr == ' ') &&
                  (ptr <= Inputfieldname+80))
                   ptr++;
            for (i=0;
                (ptr <= Inputfieldname+80) &&
                (*ptr != '\0') &&
                (*ptr != ' '); i++, ptr++)
                 buffchar??(i??) = *ptr;
            if (buffchar??(0??) != '\0')
               memcpy (mqmd.PutApplName, buffchar, 28);
            }
         if (debuglvl != 0)
            printf ("MQMD Putapplname: %s\n",mqmd.PutApplName);
         return(0);
         }

     ptr = strstr(Inputfieldname,"MQMDPUTAPPLTYPE");
     if (ptr != NULL)
       /* we have a field containing report options */
       /* nb there may be more than one - add options together */
        {
         ptr = ptr + 10;
         memset (buffnum, '\0', sizeof(buffnum));
         if (ptr <= Inputfieldname+80)
            {
            while ((*ptr == ' ') &&
                  (ptr <= Inputfieldname+80))
                   ptr++;
            for (i=0;
                (ptr <= Inputfieldname+80) &&
                (*ptr != '\0') &&
                (*ptr != ' '); i++, ptr++)
                 buffnum??(i??) = *ptr;
            if (buffnum??(0??) != '\0')
              {
                  Thisputappltype = strtol(buffnum,&endptr,10);
                  mqmd.PutApplType = mqmd.PutApplType + Thisputappltype;
              }
            }
         if (debuglvl != 0)
            printf ("MQMD Putappltype: %i\n",mqmd.PutApplType);
          return(0);
         }

     ptr = strstr(Inputfieldname,"MQMDPUTDATE");
     if (ptr != NULL)
       /* we have a field containing putdate      */
        {
         ptr = ptr + 11;
         memset (buffchar, '\0', 60);
         if (ptr <= Inputfieldname+80)
            {
            while ((*ptr == ' ') &&
                  (ptr <= Inputfieldname+80))
                   ptr++;
            for (i=0;
                (ptr <= Inputfieldname+80) &&
                (*ptr != '\0') &&
                (*ptr != ' '); i++, ptr++)
                 buffchar??(i??) = *ptr;
            if (buffchar??(0??) != '\0')
               memcpy (mqmd.PutDate, buffchar, 8);
            }
         if (debuglvl != 0)
            printf ("MQMD Putdate: %s\n",mqmd.PutDate);
         return(0);
         }

     ptr = strstr(Inputfieldname,"MQMDPUTTIME");
     if (ptr != NULL)
       /* we have a field containing puttime      */
        {
         ptr = ptr + 11;
         memset (buffchar, '\0', 60);
         if (ptr <= Inputfieldname+80)
            {
            while ((*ptr == ' ') &&
                  (ptr <= Inputfieldname+80))
                   ptr++;
            for (i=0;
                (ptr <= Inputfieldname+80) &&
                (*ptr != '\0') &&
                (*ptr != ' '); i++, ptr++)
                 buffchar??(i??) = *ptr;
            if (buffchar??(0??) != '\0')
               memcpy (mqmd.PutTime, buffchar, 8);
            }
         if (debuglvl != 0)
            printf ("MQMD Puttime: %s\n",mqmd.PutTime);
         return(0);
         }

     ptr = strstr(Inputfieldname,"MQMDAPPLORIGINDATA");
     if (ptr != NULL)
       /* we have a field containing appl origin data */
        {
         ptr = ptr + 18;
         memset (buffchar, '\0', 60);
         if (ptr <= Inputfieldname+80)
            {
            while ((*ptr == ' ') &&
                  (ptr <= Inputfieldname+80))
                   ptr++;
            for (i=0;
                (ptr <= Inputfieldname+80) &&
                (*ptr != '\0') &&
                (*ptr != ' '); i++, ptr++)
                 buffchar??(i??) = *ptr;
            if (buffchar??(0??) != '\0')
               memcpy (mqmd.ApplOriginData, buffchar, 4);
            }
         if (debuglvl != 0)
            printf ("MQMD ApplOriginData %s\n",mqmd.ApplOriginData);
         return(0);
         }

     ptr = strstr(Inputfieldname,"MQPMOOPTIONS");
     if (ptr != NULL)
       /* we have a field containing put options */
       /* nb there may be more than one - add options together */
        {
         ptr = ptr + 12;
         memset (buffnum, '\0', sizeof(buffnum));
         if (ptr <= Inputfieldname+80)
            {
            while ((*ptr == ' ') &&
                  (ptr <= Inputfieldname+80))
                   ptr++;
            for (i=0;
                (ptr <= Inputfieldname+80) &&
                (*ptr != '\0') &&
                (*ptr != ' '); i++, ptr++)
                 buffnum??(i??) = *ptr;
            if (buffnum??(0??) != '\0')
              {
                   Thisputoption = strtol(buffnum,&endptr,10);
                   mqpmo.Options = mqpmo.Options + Thisputoption;
              }
            }
         if (debuglvl != 0)
            printf ("Put message options: %i\n",mqpmo.Options);
          return(0);
         }

     ptr = strstr(Inputfieldname,"MESSAGELENGTH");
     if (ptr != NULL)
       /* we have a field containing message length */
        {
         ptr = ptr + 13;
         memset (buffnum, '\0', sizeof(buffnum));
         if (ptr <= Inputfieldname+80)
            {
            while ((*ptr == ' ') &&
                  (ptr <= Inputfieldname+80))
                   ptr++;
            for (i=0;
                (ptr <= Inputfieldname+80) &&
                (*ptr != '\0') &&
                (*ptr != ' '); i++, ptr++)
                 buffnum??(i??) = *ptr;
            if (buffnum??(0??) != '\0')
                   MessageLength = strtol(buffnum,&endptr,10);
            }
         if (debuglvl != 0)
            printf ("Message length: %i\n",MessageLength);
          return(0);
         }

     ptr = strstr(Inputfieldname,"MESSAGEDATA");
     if (ptr != NULL)
       /* we have a field containing message data */
        {
         ptr = ptr + 11;
         memset (buffchar, ' ', 60);
         if (ptr <= Inputfieldname+80)
            {
            while ((*ptr == ' ') &&
                  (ptr <= Inputfieldname+80))
                   ptr++;
            for (i=0; i<60; i++, ptr++)
                 buffchar??(i??) = *ptr;
            if (buffchar??(0??) != '\0')
              {
               memset (MessageData, ' ', 60);
               memcpy (MessageData, buffchar, 60);
              }
            }
         if (debuglvl != 0)
            printf ("Message data: %s\n",MessageData);
         return(0);
         }

     ptr = strstr(Inputfieldname,"IMSENCODING");
     if (ptr != NULL)
       /* we have a field containing IMS encoding */
        {
         ptr = ptr + 11;
         memset (buffnum, '\0', sizeof(buffnum));
         if (ptr <= Inputfieldname+80)
            {
            while ((*ptr == ' ') &&
                  (ptr <= Inputfieldname+80))
                   ptr++;
            for (i=0;
                (ptr <= Inputfieldname+80) &&
                (*ptr != '\0') &&
                (*ptr != ' '); i++, ptr++)
                 buffnum??(i??) = *ptr;
            if (buffnum??(0??) != '\0')
               IMSheader.Encoding = strtol(buffnum,&endptr,10);
            }
         if (debuglvl != 0)
            printf ("IMS encoding: %i\n",IMSheader.Encoding);
          return(0);
         }

     ptr = strstr(Inputfieldname,"IMSCODEDCHARSETID");
     if (ptr != NULL)
       /* we have a field containing IMS encoding */
        {
         ptr = ptr + 17;
         memset (buffnum, '\0', sizeof(buffnum));
         if (ptr <= Inputfieldname+80)
            {
            while ((*ptr == ' ') &&
                  (ptr <= Inputfieldname+80))
                   ptr++;
            for (i=0;
                (ptr <= Inputfieldname+80) &&
                (*ptr != '\0') &&
                (*ptr != ' '); i++, ptr++)
                 buffnum??(i??) = *ptr;
            if (buffnum??(0??) != '\0')
               IMSheader.CodedCharSetId = strtol(buffnum,&endptr,10);
            }
         if (debuglvl != 0)
            printf ("IMS CodedCharSetId: %i\n",
                     IMSheader.CodedCharSetId);
          return(0);
         }

     ptr = strstr(Inputfieldname,"IMSFORMAT");
     if (ptr != NULL)
       /* we have a field containing IMS Format */
        {
         ptr = ptr + 9;
         memset (buffchar, ' ', 60);
         if (ptr <= Inputfieldname+80)
            {
            while ((*ptr == ' ') &&
                  (ptr <= Inputfieldname+80))
                   ptr++;
            for (i=0;
                (ptr <= Inputfieldname+80) &&
                (*ptr != '\0') &&
                (*ptr != ' '); i++, ptr++)
                 buffchar??(i??) = *ptr;
            if (buffchar??(0??) != ' ')
               {
               memset (IMSheader.Format, ' ', 8);
               memcpy (IMSheader.Format, buffchar, 8);
               }
            }
         if (debuglvl != 0)
            printf ("IMS format: %s\n",IMSheader.Format);
         return(0);
         }

     ptr = strstr(Inputfieldname,"IMSFLAGS");
     if (ptr != NULL)
       /* we have a field containing IMS encoding */
        {
         ptr = ptr + 8;
         memset (buffnum, '\0', sizeof(buffnum));
         if (ptr <= Inputfieldname+80)
            {
            while ((*ptr == ' ') &&
                  (ptr <= Inputfieldname+80))
                   ptr++;
            for (i=0;
                (ptr <= Inputfieldname+80) &&
                (*ptr != '\0') &&
                (*ptr != ' '); i++, ptr++)
                 buffnum??(i??) = *ptr;
            if (buffnum??(0??) != '\0')
               IMSheader.Flags = strtol(buffnum,&endptr,10);
            }
         if (debuglvl != 0)
            printf ("IMS flags: %i\n",IMSheader.Flags);
          return(0);
         }

     ptr = strstr(Inputfieldname,"IMSLTERMOVERRIDE");
     if (ptr != NULL)
       /* we have a field containing IMS lterm */
        {
         ptr = ptr + 16;
         memset (buffchar, ' ', 60);
         if (ptr <= Inputfieldname+80)
            {
            while ((*ptr == ' ') &&
                  (ptr <= Inputfieldname+80))
                   ptr++;
            for (i=0;
                (ptr <= Inputfieldname+80) &&
                (*ptr != '\0') &&
                (*ptr != ' '); i++, ptr++)
                 buffchar??(i??) = *ptr;
            if (buffchar??(0??) != '\0')
               memset (IMSheader.LTermOverride, ' ', 8);
               memcpy (IMSheader.LTermOverride, buffchar, 8);
            }
         if (debuglvl != 0)
            printf ("IMS lterm: %s\n",IMSheader.LTermOverride);
         return(0);
         }

     ptr = strstr(Inputfieldname,"IMSMFSMAPNAME");
     if (ptr != NULL)
       /* we have a field containing MFS name  */
        {
         ptr = ptr + 13;
         memset (buffchar, ' ', 60);
         if (ptr <= Inputfieldname+80)
            {
            while ((*ptr == ' ') &&
                  (ptr <= Inputfieldname+80))
                   ptr++;
            for (i=0;
                (ptr <= Inputfieldname+80) &&
                (*ptr != '\0') &&
                (*ptr != ' '); i++, ptr++)
                 buffchar??(i??) = *ptr;
            if (buffchar??(0??) != '\0')
               memset (IMSheader.MFSMapName, ' ', 8);
               memcpy (IMSheader.MFSMapName, buffchar, 8);
            }
         if (debuglvl != 0)
            printf ("IMS MFSname: %s\n",IMSheader.MFSMapName);
         return(0);
         }

     ptr = strstr(Inputfieldname,"IMSREPLYTOFORMAT");
     if (ptr != NULL)
       /* we have a field containing reply to format */
        {
         ptr = ptr + 16;
         memset (buffchar, '\0', 60);
         if (ptr <= Inputfieldname+80)
            {
            while ((*ptr == ' ') &&
                  (ptr <= Inputfieldname+80))
                   ptr++;
            for (i=0;
                (ptr <= Inputfieldname+80) &&
                (*ptr != '\0') &&
                (*ptr != ' '); i++, ptr++)
                 buffchar??(i??) = *ptr;
            if (buffchar??(0??) != '\0')
               memset (IMSheader.ReplyToFormat, ' ', 8);
               memcpy (IMSheader.ReplyToFormat, buffchar, 8);
            }
         if (debuglvl != 0)
          printf ("IMS Replyto format: %s\n",IMSheader.ReplyToFormat);
         return(0);
         }

     ptr = strstr(Inputfieldname,"IMSAUTHENTICATOR");
     if (ptr != NULL)
       /* we have a field containing passticket      */
        {
         ptr = ptr + 16;
         memset (buffchar, '\0', 60);
         if (ptr <= Inputfieldname+80)
            {
            while ((*ptr == ' ') &&
                  (ptr <= Inputfieldname+80))
                   ptr++;
            for (i=0;
                (ptr <= Inputfieldname+80) &&
                (*ptr != '\0') &&
                (*ptr != ' '); i++, ptr++)
                 buffchar??(i??) = *ptr;
            if (buffchar??(0??) != '\0')
               memset (IMSheader.Authenticator, ' ', 8);
               memcpy (IMSheader.Authenticator, buffchar,
                       strlen(buffchar) );
            }
         if (debuglvl != 0)
          printf ("IMS Authenticator: %s\n",IMSheader.Authenticator);
         return(0);
         }

     ptr = strstr(Inputfieldname,"IMSTXNINSTANCEID");
     if (ptr != NULL)
       /* we have a field containing passticket      */
        {
         ptr = ptr + 16;
         memset (buffchar, '\0', 60);
         if (ptr <= Inputfieldname+80)
            {
            while ((*ptr == ' ') &&
                  (ptr <= Inputfieldname+80))
                   ptr++;
            for (i=0;
                (ptr <= Inputfieldname+80) &&
                (*ptr != '\0') &&
                (*ptr != ' '); i++, ptr++)
                 buffchar??(i??) = *ptr;
            if (buffchar??(0??) != '\0')
               {
               memcpy (IMSheader.TranInstanceId, buffchar, 8);
               memset (IMSheader.TranInstanceId + 8, ' ', 8);
               }
            }
         if (debuglvl != 0)
          printf ("IMS Instance id: %s\n",IMSheader.TranInstanceId);
         return(0);
         }

     ptr = strstr(Inputfieldname,"IMSTRANSTATE");
     if (ptr != NULL)
       /* we have a field containing tran state */
        {
         ptr = ptr + 12;
         memset (buffchar, '\0', 60);
         if (ptr <= Inputfieldname+80)
            {
            while ((*ptr == ' ') &&
                  (ptr <= Inputfieldname+80))
                   ptr++;
            for (i=0;
                (ptr <= Inputfieldname+80) &&
                (*ptr != '\0') &&
                (*ptr != ' '); i++, ptr++)
                 buffchar??(i??) = *ptr;
            if (buffchar??(0??) != '\0')
               IMSheader.TranState = buffchar??(0??);
            }
         if (debuglvl != 0)
          printf ("IMS tran state: %c\n",IMSheader.TranState);
         return(0);
         }

     ptr = strstr(Inputfieldname,"IMSCOMMITMODE");
     if (ptr != NULL)
       /* we have a field containing commit mode */
        {
         ptr = ptr + 13;
         memset (buffchar, '\0', 60);
         if (ptr <= Inputfieldname+80)
            {
            while ((*ptr == ' ') &&
                  (ptr <= Inputfieldname+80))
                   ptr++;
            for (i=0;
                (ptr <= Inputfieldname+80) &&
                (*ptr != '\0') &&
                (*ptr != ' '); i++, ptr++)
                 buffchar??(i??) = *ptr;
            if (buffchar??(0??) != '\0')
               IMSheader.CommitMode = buffchar??(0??);
            }
         if (debuglvl != 0)
          printf ("IMS Commit mode: %c\n",IMSheader.CommitMode);
         return(0);
         }

     ptr = strstr(Inputfieldname,"IMSSECURITYSCOPE");
     if (ptr != NULL)
       /* we have a field containing reserved data */
        {
         ptr = ptr + 16;
         memset (buffchar, '\0', 60);
         if (ptr <= Inputfieldname+80)
            {
            while ((*ptr == ' ') &&
                  (ptr <= Inputfieldname+80))
                   ptr++;
            for (i=0;
                (ptr <= Inputfieldname+80) &&
                (*ptr != '\0') &&
                (*ptr != ' '); i++, ptr++)
                 buffchar??(i??) = *ptr;
            if (buffchar??(0??) != '\0')
               IMSheader.SecurityScope = buffchar??(0??);
            }
         if (debuglvl != 0)
          printf ("IMS security scope: %c\n",IMSheader.SecurityScope);
         return(0);
         }

     ptr = strstr(Inputfieldname,"IMSRESERVED");
     if (ptr != NULL)
       /* we have a field containing reserved data */
        {
         ptr = ptr + 11;
         memset (buffchar, '\0', 60);
         if (ptr <= Inputfieldname+80)
            {
            while ((*ptr == ' ') &&
                  (ptr <= Inputfieldname+80))
                   ptr++;
            for (i=0;
                (ptr <= Inputfieldname+80) &&
                (*ptr != '\0') &&
                (*ptr != ' '); i++, ptr++)
                 buffchar??(i??) = *ptr;
            if (buffchar??(0??) != '\0')
               IMSheader.Reserved = buffchar??(0??);
            }
         if (debuglvl != 0)
          printf ("IMS reserved: %c\n",IMSheader.Reserved);
         return(0);
         }

     ptr = strstr(Inputfieldname,"IMSLL");
     if (ptr != NULL)
       /* we have a field containing message length */
        {
         ptr = ptr + 5;
         memset (buffnum, '\0', sizeof(buffnum));
         if (ptr <= Inputfieldname+80)
            {
            while ((*ptr == ' ') &&
                  (ptr <= Inputfieldname+80))
                   ptr++;
            for (i=0;
                (ptr <= Inputfieldname+80) &&
                (*ptr != '\0') &&
                (*ptr != ' '); i++, ptr++)
                 buffnum??(i??) = *ptr;
            if (buffnum??(0??) != '\0')
              {
                   IMSLLlong = strtol(buffnum,&endptr,10);
                   IMSLL = IMSLLlong;
              }
            }
         if (debuglvl != 0)
          printf ("IMS ll: %i\n",IMSLL);
          return(0);
        }

     ptr = strstr(Inputfieldname,"IMSZZ");
     if (ptr != NULL)
       /* we have a field containing zz bytes */
        {
         ptr = ptr + 5;
         memset (buffnum, '\0', sizeof(buffnum));
         if (ptr <= Inputfieldname+80)
            {
            while ((*ptr == ' ') &&
                  (ptr <= Inputfieldname+80))
                   ptr++;
            for (i=0;
                (ptr <= Inputfieldname+80) &&
                (*ptr != '\0') &&
                (*ptr != ' '); i++, ptr++)
                 buffnum??(i??) = *ptr;
            if (buffnum??(0??) != '\0')
              {
                   IMSZZlong = strtol(buffnum,&endptr,10);
                   IMSZZ = IMSZZlong;
              }
            }
         if (debuglvl != 0)
             printf ("IMS zz: %i\n",IMSZZ);
          return(0);

        }
        return(0);
}